Module jet

Module jet 

Simplicity jets

Jets are special nodes that read a value, process it internally, and write an output value. This evaluation happens in a black-box manner: In terms of the Bit Machine, it is a one-step process.

In practice, jets call foreign C code that is equivalent to some Simplicity DAG. This speeds up evaluation tremendously. Equivalence of C and Simplicity is proved using the Verified Software Toolchain. Programs are also smaller in size because jets replace large, equivalent Simplicity DAGs.

Re-exports

pub use self::core::CoreEnv;
pub use crate::jet::bitcoin::BitcoinEnv;
pub use elements::ElementsTxEnv;

Modules

bitcoin
core
elements
type_name
Manual type specification

Structs

JetFailed
Generic error that a jet failed during its execution.

Enums

Bitcoin
The Bitcoin jet family.
Core
The Core jet family.
Elements
The Elements jet family.

Traits

DynJet
Lets Box<dyn Jet> work with Clone, Eq, Ord, and Hash.
Jet
Family of jets that share an encoding scheme and execution environment.
JetEnvironment
An environment for jets to read.