Jet

Trait Jet 

pub trait Jet:
    DynJet
    + Debug
    + Display
    + Send
    + Sync
    + 'static {
    // Required methods
    fn cmr(&self) -> Cmr;
    fn source_ty(&self) -> TypeName;
    fn target_ty(&self) -> TypeName;
    fn encode(&self, w: &mut BitWriter<&mut dyn Write>) -> Result<usize>;
    fn decode<I: Iterator<Item = u8>>(
        bits: &mut BitIter<I>,
    ) -> Result<Self, Error>
       where Self: Sized;
    fn cost(&self) -> Cost;
    fn parse(s: &str) -> Result<Self, Error>
       where Self: Sized;
}
Family of jets that share an encoding scheme and execution environment.

Jets are single nodes that read an input, process it internally using foreign C code (black box), and produce an output. Jets may read values from their environment.

Jets are always leaves in a Simplicity DAG.

Required Methods

fn cmr(&self) -> Cmr

Return the CMR of the jet.

fn source_ty(&self) -> TypeName

Return the source type of the jet.

fn target_ty(&self) -> TypeName

Return the target type of the jet.

fn encode(&self, w: &mut BitWriter<&mut dyn Write>) -> Result<usize>

Encode the jet to bits.

fn decode<I: Iterator<Item = u8>>(bits: &mut BitIter<I>) -> Result<Self, Error>
where Self: Sized,

Decode a jet from bits.

fn cost(&self) -> Cost

Return the cost of the jet.

fn parse(s: &str) -> Result<Self, Error>
where Self: Sized,

Parse a jet from a string.