pub struct Construct<'brand> { /* private fields */ }Trait Implementations
Sourceimpl<'brand> Marker for Construct<'brand>
impl<'brand> Marker for Construct<'brand>
Sourcetype CachedData = ConstructData<'brand>
type CachedData = ConstructData<'brand>
Precomputed data about the node, such as its type arrow or various Merkle roots.
Sourcetype Disconnect = Option<Arc<Node<Construct<'brand>>>>
type Disconnect = Option<Arc<Node<Construct<'brand>>>>
Type of disconnect data attached to DAGs of this node type.
Sourcetype SharingId = ConstructId
type SharingId = ConstructId
A type which uniquely identifies a node, for purposes of sharing
during iteration over the DAG.
Sourcefn compute_sharing_id(_: Cmr, _: &ConstructData<'brand>) -> Option<ConstructId>
fn compute_sharing_id(_: Cmr, _: &ConstructData<'brand>) -> Option<ConstructId>
Yields the sharing ID for a given type, starting from its CMR and its cached data. Read more
Sourceimpl<'brand> Ord for Construct<'brand>
impl<'brand> Ord for Construct<'brand>
1.21.0 (const: unstable) · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
Compares and returns the maximum of two values. Read more
Sourceimpl<'brand> PartialEq for Construct<'brand>
impl<'brand> PartialEq for Construct<'brand>
Sourceimpl<'brand> PartialOrd for Construct<'brand>
impl<'brand> PartialOrd for Construct<'brand>
impl<'brand> Copy for Construct<'brand>
impl<'brand> Eq for Construct<'brand>
impl<'brand> StructuralPartialEq for Construct<'brand>