pub struct Redeem { /* private fields */ }Trait Implementations
Sourceimpl<W: Iterator<Item = Value>> Converter<Commit, Redeem> for SimpleFinalizer<W>
impl<W: Iterator<Item = Value>> Converter<Commit, Redeem> for SimpleFinalizer<W>
Sourcetype Error = FinalizeError
type Error = FinalizeError
The error type returned by the methods on this trait.
Sourcefn convert_witness(
&mut self,
data: &PostOrderIterItem<&CommitNode>,
_: &NoWitness,
) -> Result<Value, Self::Error>
fn convert_witness( &mut self, data: &PostOrderIterItem<&CommitNode>, _: &NoWitness, ) -> Result<Value, Self::Error>
For witness nodes, this method is called first to attach witness data to
the node. Read more
Sourcefn convert_disconnect(
&mut self,
_: &PostOrderIterItem<&CommitNode>,
_: Option<&Arc<RedeemNode>>,
_: &NoDisconnect,
) -> Result<Arc<RedeemNode>, Self::Error>
fn convert_disconnect( &mut self, _: &PostOrderIterItem<&CommitNode>, _: Option<&Arc<RedeemNode>>, _: &NoDisconnect, ) -> Result<Arc<RedeemNode>, Self::Error>
For disconnect nodes, this method is called first to attach a disconnected
expression to the node. Read more
Sourcefn convert_data(
&mut self,
data: &PostOrderIterItem<&CommitNode>,
inner: Inner<&Arc<RedeemNode>, &Arc<RedeemNode>, &Value>,
) -> Result<Arc<RedeemData>, Self::Error>
fn convert_data( &mut self, data: &PostOrderIterItem<&CommitNode>, inner: Inner<&Arc<RedeemNode>, &Arc<RedeemNode>, &Value>, ) -> Result<Arc<RedeemData>, Self::Error>
This method is called for every node, after
Self::convert_witness or
Self::prune_case, if either is applicable. Read moreSourcefn visit_node(&mut self, _data: &PostOrderIterItem<&Node<N>>)
fn visit_node(&mut self, _data: &PostOrderIterItem<&Node<N>>)
This method is called on every node, to inform the
Converter about the
state of the iterator. Read moreSourceimpl Marker for Redeem
impl Marker for Redeem
Sourcetype CachedData = Arc<RedeemData>
type CachedData = Arc<RedeemData>
Precomputed data about the node, such as its type arrow or various Merkle roots.
Sourcetype Disconnect = Arc<Node<Redeem>>
type Disconnect = Arc<Node<Redeem>>
Type of disconnect data attached to DAGs of this node type.
Sourcetype SharingId = Ihr
type SharingId = Ihr
A type which uniquely identifies a node, for purposes of sharing
during iteration over the DAG.
Sourcefn compute_sharing_id(_: Cmr, cached_data: &Arc<RedeemData>) -> Option<Ihr>
fn compute_sharing_id(_: Cmr, cached_data: &Arc<RedeemData>) -> Option<Ihr>
Yields the sharing ID for a given type, starting from its CMR and its cached data. Read more
Sourceimpl Ord for Redeem
impl Ord for Redeem
1.21.0 (const: unstable) · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
Compares and returns the maximum of two values. Read more
Sourceimpl PartialOrd for Redeem
impl PartialOrd for Redeem
impl Copy for Redeem
impl Eq for Redeem
impl StructuralPartialEq for Redeem