pub struct SimpleFinalizer<W: Iterator<Item = Value>> { /* private fields */ }Expand description
Basic finalizer which converts a super::CommitNode to a super::RedeemNode
by attaching witness data from an iterator.
Does not do any type-checking and may attach an invalid witness to a combinator.
If it encounters a disconnect node, it simply returns an error.
This finalizer should not be used in production.
It introduces default values (Value::zero) to handle missing witness data,
which might trigger unexpected spending paths.
Implementations