pub trait PruneTracker: ExecTracker {
// Required methods
fn contains_left(&self, ihr: Ihr) -> bool;
fn contains_right(&self, ihr: Ihr) -> bool;
}Required Methods
Sourcefn contains_left(&self, ihr: Ihr) -> bool
fn contains_left(&self, ihr: Ihr) -> bool
Returns true if the left branch of the of the Case node with the IHR ihr was taken.
Sourcefn contains_right(&self, ihr: Ihr) -> bool
fn contains_right(&self, ihr: Ihr) -> bool
Returns true if the right branch of the of the Case node with the IHR ihr was taken.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".