pub enum WitnessOrHole {
Witness,
TypedHole(Arc<str>),
}Expand description
For named construct nodes, we abuse the witness combinator to support typed holes.
We do this because witness nodes have free source and target arrows, and
allow us to store arbitrary data in them using the generic witness type of
the node. Holes are characterized entirely by their source and target type,
just like witnesses, and are labelled by their name.
Variants
Witness
This witness is an actual witness combinator (with no witness data attached, that comes later)
TypedHole(Arc<str>)
This is a typed hole, with the given name.
Implementations
Sourceimpl WitnessOrHole
impl WitnessOrHole
pub fn shallow_clone(&self) -> Self