pub trait Jet:
DynJet
+ Debug
+ Display
+ Send
+ Sync
+ 'static {
// Required methods
fn cmr(&self) -> Cmr;
fn source_ty(&self) -> TypeName;
fn target_ty(&self) -> TypeName;
fn encode(&self, w: &mut BitWriter<&mut dyn Write>) -> Result<usize>;
fn decode<I: Iterator<Item = u8>>(
bits: &mut BitIter<I>,
) -> Result<Self, Error>
where Self: Sized;
fn cost(&self) -> Cost;
fn parse(s: &str) -> Result<Self, Error>
where Self: Sized;
}Expand description
Family of jets that share an encoding scheme and execution environment.
Jets are single nodes that read an input, process it internally using foreign C code (black box), and produce an output. Jets may read values from their environment.
Jets are always leaves in a Simplicity DAG.
Required Methods